using EPiServer.Commerce.Order;
using EPiServer.ServiceLocation;
using Mediachase.Commerce.Engine;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.WorkflowCompatibility;
using System;
using System.Linq;

namespace Mediachase.Commerce.Workflow.Activities.Cart
{
    /// <summary>
    /// This activity handles processing different types of payments. It will call the appropriate 
    /// payment handler configured in the database and raise exceptions if something goes wrong.
    /// It also deals with removing sensitive data for credit card types of payments depending on the 
    /// configuration settings.
    /// </summary>
    public class ProcessPaymentActivity : CartActivityBase
    {
        private string _processingPaymentEvent = "ProcessingPayment";
        private string _processedPaymentEvent = "ProcessedPayment";
        private Injected<IPaymentProcessor> _paymentProcessor;

        // Define private constants for the Validation Errors 
        private const int TotalPaymentMismatch = 1;

        /// <summary>
        /// Gets or sets the payment.
        /// </summary>
        /// <value>The payment.</value>
        [ActivityFlowContextProperty]
        public IPayment Payment { get; set; }

        /// <summary>
        /// Gets or sets the shipment.
        /// </summary>
        /// <value>The shipment.</value>
        [ActivityFlowContextProperty]
        public Shipment Shipment { get; set; }

        /// <summary>
        /// Executes the specified execution context.
        /// </summary>
        /// <param name="executionContext">The execution context.</param>
        /// <returns></returns>
        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            // Raise the ProcessingPaymentEvent event to the parent workflow
            RaiseEvent(_processingPaymentEvent, EventArgs.Empty);

            // Validate the properties at runtime
            ValidateRuntime();

            // Process payment now
            ProcessPayment();

            // Raise the ProcessedPaymentEvent event to the parent workflow
            RaiseEvent(_processedPaymentEvent, EventArgs.Empty);

            // Retun the closed status indicating that this activity is complete.
            return ActivityExecutionStatus.Closed;
        }

        /// <summary>
        /// Validates the order properties.
        /// </summary>
        /// <param name="validationErrors">The validation errors.</param>
        protected override void ValidateOrderProperties(ValidationErrorCollection validationErrors)
        {
            base.ValidateOrderProperties(validationErrors);

            var order = (IOrderGroup)OrderGroup;

            if (Payment != null)
            {
                return;
            }

            // Cycle through all Order Forms and check total, it should be equal to total of all payments.;
            var payments = order.Forms.SelectMany(f => f.Payments);
            var paymentTotal = payments.Sum(p => p.Amount);
            var capturedPaymentTotal = payments
                    .Where(p => p.TransactionType != TransactionType.Authorization.ToString() && 
                                p.TransactionType != TransactionType.Credit.ToString())
                    .Sum(p => p.Amount);

            if (paymentTotal < OrderGroup.Total)
            {
                validationErrors.Add(new ValidationError("The payment total and the order total do not match. Please adjust your payment.", TotalPaymentMismatch, false));
            }

            if (capturedPaymentTotal > OrderGroup.Total)
            {
                AddWarningSafe(Warnings, $"PaymentTotal-{OrderGroup.Id}", "Payment total is greater than order total.");
            }
        }

        /// <summary>
        /// Processes the payment.
        /// </summary>
        private void ProcessPayment()
        {
            var order = (IOrderGroup)OrderGroup;
            // If total is 0, we do not need to proceed
            if (OrderGroup.Total == 0 || order is IPaymentPlan)
            {
                return;
            }

            foreach (var payment in order.Forms.SelectMany(f => f.Payments))
            {
                if (Payment != null && !Payment.Equals(payment))
                {
                    continue;
                }

                _paymentProcessor.Service.ProcessPayment(order, payment, Shipment);
            }
        }
    }
}
