using EPiServer;
using EPiServer.Commerce.Catalog.ContentTypes;
using EPiServer.Commerce.Marketing;
using EPiServer.Commerce.Order;
using EPiServer.Commerce.Order.Internal;
using EPiServer.ServiceLocation;
using Mediachase.Commerce.Catalog;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.WorkflowCompatibility;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Mediachase.Commerce.Workflow.Activities
{
    /// <summary>
    /// Calculate discounts using the new promotion engine
    /// </summary>
    public class CalculateDiscountsActivity : OrderGroupActivityBase
    {
        /// <summary>
        /// Executes the specified execution context.
        /// </summary>
        /// <param name="executionContext">The execution context.</param>
        /// <returns></returns>
        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            ValidateRuntime();
            OrderGroup.ApplyDiscounts();
            OrderGroup.UpdateInventoryOrRemoveLineItems((item, issue) => { });

            var order = (IOrderGroup) OrderGroup;
            var entryCodes = GetAffectedEntryCodes(order);
            var codeContentLinkMap = ReferenceConverter.GetContentLinks(entryCodes);

            UpdateGiftLineItems();

            var itemInOrderForms = OrderGroup.OrderForms.SelectMany(x => x.LineItems);
            var itemInShipments = order.Forms.SelectMany(x => x.Shipments).SelectMany(x => x.LineItems);
            foreach (var contentLink in codeContentLinkMap.Values)
            {
                var item = ContentLoader.Get<EntryContentBase>(contentLink);
                if (item == null)
                {
                    continue;
                }

                foreach (var lineItem in itemInShipments.Where(x => x.Code.Equals(item.Code)))
                {
                    var metaItem = itemInOrderForms.FirstOrDefault(x => x.LineItemId == lineItem.LineItemId);
                    if (metaItem != null)
                    {
                        metaItem.SetEntryDiscountValue(lineItem.GetEntryDiscountValue());
                        metaItem.SetOrderDiscountValue(lineItem.GetOrderDiscountValue());
                    }
                }
            }

            return ActivityExecutionStatus.Closed;
        }

        private void UpdateGiftLineItems()
        {
            if (!OrderGroup.GetAllLineItems().Any(item => item.IsGift))
            {
                return;
            }

            UpdateGiftLineItemIndexQuantity();
        }

        private void UpdateGiftLineItemIndexQuantity()
        {
            foreach (OrderForm orderForm in OrderGroup.OrderForms)
            {
                foreach (Shipment shipment in orderForm.Shipments)
                {
                    foreach (var giftItem in shipment.LineItems.Where(item => item.IsGift))
                    {
                        var index = GetLineItemIndex(orderForm, giftItem);
                        if (index > -1 && shipment.LineItemIndexes.Contains(index.ToString()))
                        {
                            shipment.SetLineItemQuantity(index, giftItem.Quantity);
                        }
                    }
                }
            }
        }

        private int GetLineItemIndex(OrderForm form, ILineItem lineItem)
        {
            for (int i = 0; i < form.LineItems.Count; i++)
            {
                if (form.LineItems[i].LineItemId == lineItem.LineItemId)
                {
                    return i;
                }
            }
            return -1;
        }

        private static IEnumerable<string> GetAffectedEntryCodes(IOrderGroup order)
        {
            if (HasOrderPromotionWithSavedAmount(order))
            {
                return GetAllEntryCodesFromOrder(order);
            }

            return GetAffectedEntryCodesFromEntryPromotions(order);
        }

        private static bool HasOrderPromotionWithSavedAmount(IOrderGroup order)
        {
            return order.Forms.SelectMany(f => f.Promotions).Any(x => x.OrderForm.SavedAmount > 0);
        }

        private static IEnumerable<string> GetAffectedEntryCodesFromEntryPromotions(IOrderGroup order)
        {
            return order.Forms.SelectMany(f => f.Promotions).SelectMany(p => p.Entries).Select(i => i.EntryCode);
        }

        private static IEnumerable<string> GetAllEntryCodesFromOrder(IOrderGroup order)
        {
            return order.Forms.SelectMany(x => x.Shipments).SelectMany(x => x.LineItems).Select(x => x.Code);
        }
    }
}
