using EPiServer.Commerce.Order;
using EPiServer.ServiceLocation;
using Mediachase.Commerce.Markets;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.WorkflowCompatibility;
using Mediachase.MetaDataPlus;
using System.Linq;

namespace Mediachase.Commerce.Workflow.Activities
{
    public class CalculateTotalsActivity : OrderGroupActivityBase
    {
        private Injected<IOrderGroupCalculator> OrderGroupCalculator = default;
        private Injected<IOrderFormCalculator> OrderFormCalculator = default;
        private Injected<IShippingCalculator> ShippingCalculator = default;
        private Injected<ILineItemCalculator> LineItemCalculator = default;
        private Injected<IMarketService> MarketService = default;

        /// <summary>
        /// Called by the workflow runtime to execute an activity.
        /// </summary>
        /// <param name="executionContext">The <see cref="T:Mediachase.Commerce.WorkflowCompatibility.ActivityExecutionContext"/> to associate with this <see cref="T:Mediachase.Commerce.WorkflowCompatibility.Activity"/> and execution.</param>
        /// <returns>
        /// The <see cref="T:Mediachase.Commerce.WorkflowCompatibility.ActivityExecutionStatus"/> of the run task, which determines whether the activity remains in the executing state, or transitions to the closed state.
        /// </returns>
        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            // Validate the properties at runtime
            ValidateRuntime();

            // Calculate order totals
            CalculateTotals();

            // Retun the closed status indicating that this activity is complete.
            return ActivityExecutionStatus.Closed;
        }

        /// <summary>
        /// Calculates the totals.
        /// </summary>
        private void CalculateTotals()
        {
            if (OrderGroup.ObjectState == MetaObjectState.Deleted)
            {
                return;
            }

            // Get the property, since it is expensive process, make sure to get it once
            var order = (IOrderGroup)OrderGroup;
            var currency = order.Currency;
            var market = MarketService.Service.GetMarket(order.MarketId);

            foreach (OrderForm form in OrderGroup.OrderForms)
            {
                CalculateTotalsOrderForms(form, market, currency);
            }

            OrderGroup.SubTotal = OrderGroupCalculator.Service.GetSubTotal(OrderGroup);
            OrderGroup.ShippingTotal = OrderGroupCalculator.Service.GetShippingSubTotal(OrderGroup);
            OrderGroup.Total = OrderGroupCalculator.Service.GetTotal(OrderGroup);
            OrderGroup.TaxTotal = OrderGroupCalculator.Service.GetTaxTotal(OrderGroup);
            OrderGroup.HandlingTotal = OrderGroupCalculator.Service.GetHandlingTotal(OrderGroup);
        }

        /// <summary>
        /// Calculates the totals order forms.
        /// </summary>
        /// <param name="form">The form.</param>
        /// <param name="market">The market.</param>
        /// <param name="currency">The currency.</param>
        private void CalculateTotalsOrderForms(OrderForm form, IMarket market, Currency currency)
        {
            foreach (var item in form.LineItems.Where(x => x.ObjectState != MetaObjectState.Deleted))
            {
                item.ExtendedPrice = LineItemCalculator.Service.GetExtendedPrice(item, currency);
            }

            foreach (var shipment in form.Shipments.Where(x => x.ObjectState != MetaObjectState.Deleted))
            {
                shipment.SubTotal = ShippingCalculator.Service.GetShippingItemsTotal(shipment, currency);
            }

            form.ShippingTotal = OrderFormCalculator.Service.GetShippingSubTotal(form, market, currency);
            form.DiscountAmount = OrderFormCalculator.Service.GetDiscountTotal(form, currency);
            form.SubTotal = OrderFormCalculator.Service.GetSubTotal(form, currency);
            form.Total = OrderFormCalculator.Service.GetTotal(form, market, currency);
            form.TaxTotal = OrderFormCalculator.Service.GetTaxTotal(form, market, currency);

            form.UpdatePaymentTotals();
        }
    }
}