﻿using EPiServer;
using EPiServer.Commerce.Catalog.ContentTypes;
using EPiServer.Commerce.Internal;
using EPiServer.Commerce.Order;
using EPiServer.ServiceLocation;
using Mediachase.Commerce.Catalog;
using Mediachase.Commerce.Customers;
using Mediachase.Commerce.Engine;
using Mediachase.Commerce.Inventory;
using Mediachase.Commerce.InventoryService;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.Pricing;
using Mediachase.Commerce.WorkflowCompatibility;
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Globalization;
using System.Linq;

namespace Mediachase.Commerce.Workflow.Activities
{
    public abstract class OrderGroupActivityBase : Activity
    {
        protected IInventoryService InventoryService => ServiceLocator.Current.GetInstance<IInventoryService>();
        protected IWarehouseRepository WarehouseRepository => ServiceLocator.Current.GetInstance<IWarehouseRepository>();
        protected ReferenceConverter ReferenceConverter => ServiceLocator.Current.GetInstance<ReferenceConverter>();
        protected IContentLoader ContentLoader => ServiceLocator.Current.GetInstance<IContentLoader>();
        protected DateTime SafeBeginningOfTime => new DateTime(1900, 1, 1, 0, 0, 0, DateTimeKind.Utc);
        protected IPriceService PriceService => ServiceLocator.Current.GetInstance<IPriceService>();
        protected InventoryRequester InventoryRequester => ServiceLocator.Current.GetInstance<InventoryRequester>();

        /// <summary>
        /// Gets or sets the order group.
        /// </summary>
        /// <value>The order group.</value>
        [ActivityFlowContextProperty]
        public OrderGroup OrderGroup { get; set; }

        /// <summary>
        /// Gets or sets the warnings.
        /// </summary>
        /// <value>The warnings.</value>
        [ActivityFlowContextProperty]
        public StringDictionary Warnings { get; set; }

        /// <summary>
        /// Validates the runtime.
        /// </summary>
        /// <returns></returns>
        protected virtual bool ValidateRuntime()
        {
            // Create a new collection for storing the validation errors
            var validationErrors = new ValidationErrorCollection();

            // Validate the Order Properties
            ValidateOrderProperties(validationErrors);

            // Validate properties
            ValidateProperties(validationErrors);

            // Raise an exception if we have ValidationErrors
            if (validationErrors.HasErrors)
            {
                var validationErrorsMessage = string.Join(string.Empty, validationErrors.Select(error => $"Validation Error: Number {error.ErrorNumber} - '{error.ErrorText}' \n"));

                // Throw a new exception with the validation errors.
                throw new WorkflowValidationFailedException(validationErrorsMessage, validationErrors);

            }
            // If we made it this far, then the data must be valid.
            return true;
        }

        /// <summary>
        /// Validates the order properties.
        /// </summary>
        /// <param name="validationErrors">The validation errors.</param>
        protected virtual void ValidateOrderProperties(ValidationErrorCollection validationErrors)
        {
            // Validate the To property
            if (OrderGroup == null)
            {
                var validationError = ValidationError.GetNotSetValidationError("OrderGroup");
                validationErrors.Add(validationError);
            }
        }

        protected virtual Money? GetItemPrice(ILineItem lineItem, CustomerContact customerContact, string billingCurrency, MarketId marketId)
        {
            var _mapUserKey = new MapUserKey();
            var now = DateTime.UtcNow;
            var currency = new Currency(billingCurrency);
            var customerPricing = new List<CustomerPricing>();
            customerPricing.Add(CustomerPricing.AllCustomers);
            if (customerContact != null)
            {
                var userKey = _mapUserKey.ToUserKey(customerContact.UserId);
                if (userKey != null && !string.IsNullOrWhiteSpace(userKey.ToString()))
                {
                    customerPricing.Add(new CustomerPricing(CustomerPricing.PriceType.UserName, userKey.ToString()));
                }

                if (!string.IsNullOrEmpty(customerContact.EffectiveCustomerGroup))
                {
                    customerPricing.Add(new CustomerPricing(CustomerPricing.PriceType.PriceGroup, customerContact.EffectiveCustomerGroup));
                }
            }


            PriceFilter priceFilter = new PriceFilter()
            {
                Currencies = new List<Currency>() { currency },
                Quantity = lineItem.Quantity,
                CustomerPricing = customerPricing,
                ReturnCustomerPricing = false // just want one value
            };

            // Get the lowest price among all the prices matching the parameters
            var priceValue = PriceService
                                    .GetPrices(marketId, now, new CatalogKey(lineItem.Code), priceFilter)
                                    .OrderBy(pv => pv.UnitPrice)
                                    .FirstOrDefault();

            if (priceValue != null)
            {
                return priceValue.UnitPrice;
            }

            if (lineItem.PlacedPrice != 0)
            {
                return new Money(lineItem.PlacedPrice, currency);
            }

            return null;
        }

        protected void AddWarningSafe(StringDictionary warnings, string key, string value)
        {
            string uniqueKey, uniqueKeyPrefix = key + '-';

            int counter = 1;
            do
            {
                string suffix = counter.ToString(CultureInfo.InvariantCulture);
                uniqueKey = uniqueKeyPrefix + suffix;
                ++counter;
            }
            while (warnings.ContainsKey(uniqueKey));

            warnings.Add(uniqueKey, value);
        }

        protected void AddValidationIssues(IDictionary<ILineItem, IList<ValidationIssue>> issues, ILineItem lineItem, ValidationIssue issue)
        {
            if (!issues.ContainsKey(lineItem))
            {
                issues.Add(lineItem, new List<ValidationIssue>());
            }

            if (!issues[lineItem].Contains(issue))
            {
                issues[lineItem].Add(issue);
            }
        }

        protected void AddWarningMessages(IDictionary<ILineItem, IList<ValidationIssue>> validationIssueCollections)
        {
            foreach (var validationIssue in validationIssueCollections)
            {
                var lineItemCode = validationIssue.Key.Code;
                foreach (var issue in validationIssue.Value)
                {
                    switch (issue)
                    {
                        case ValidationIssue.None:
                            break;

                        case ValidationIssue.RemovedGiftDueToInsufficientQuantityInInventory:
                            AddWarningSafe(Warnings,
                                ValidationIssue.RemovedGiftDueToInsufficientQuantityInInventory.ToString(),
                                $"The gift {lineItemCode} has not enough quantity in stock and was removed from your cart.");
                            break;

                        case ValidationIssue.CannotProcessDueToMissingOrderStatus:
                            AddWarningSafe(Warnings,
                                ValidationIssue.CannotProcessDueToMissingOrderStatus.ToString(),
                                $"Cannot process product {lineItemCode} because of missing order status.");
                            break;

                        case ValidationIssue.RemovedDueToCodeMissing:
                            AddWarningSafe(Warnings,
                                ValidationIssue.RemovedDueToCodeMissing.ToString(),
                                $"The product {lineItemCode} was removed from your cart because the catalog entry code that maps to the line item has been removed or changed.");
                            break;

                        case ValidationIssue.RemovedDueToNotAvailableInMarket:
                            AddWarningSafe(Warnings,
                                ValidationIssue.RemovedDueToNotAvailableInMarket.ToString(),
                                $"The product {lineItemCode} was removed from your cart because it is not available in your market.");
                            break;

                        case ValidationIssue.RemovedDueToInactiveWarehouse:
                            AddWarningSafe(Warnings,
                                ValidationIssue.RemovedDueToInactiveWarehouse.ToString(),
                                $"The product {lineItemCode} was removed from your cart because the selected warehouse is inactive.");
                            break;

                        case ValidationIssue.RemovedDueToMissingInventoryInformation:
                            AddWarningSafe(Warnings,
                                ValidationIssue.RemovedDueToMissingInventoryInformation.ToString(),
                                $"The product {lineItemCode} was removed from your cart due to missing inventory information.");
                            break;

                        case ValidationIssue.RemovedDueToUnavailableCatalog:
                            AddWarningSafe(Warnings,
                                ValidationIssue.RemovedDueToUnavailableCatalog.ToString(),
                                $"The product {lineItemCode} was removed from your cart because the catalog of this entry is not available.");
                            break;

                        case ValidationIssue.RemovedDueToUnavailableItem:
                            AddWarningSafe(Warnings,
                                ValidationIssue.RemovedDueToUnavailableItem.ToString(),
                                $"The product {lineItemCode} is not available in store and was removed from your cart.");
                            break;

                        case ValidationIssue.RemovedDueToInsufficientQuantityInInventory:
                            AddWarningSafe(Warnings,
                                ValidationIssue.RemovedGiftDueToInsufficientQuantityInInventory.ToString(),
                                $"The product {lineItemCode} is sold out and was removed from your cart.");
                            break;

                        case ValidationIssue.RemovedDueToInvalidPrice:
                            AddWarningSafe(Warnings,
                                ValidationIssue.RemovedDueToInvalidPrice.ToString(),
                                $"The product {lineItemCode} does not have a valid price and was removed from your cart.");
                            break;

                        case ValidationIssue.RemovedDueToInvalidMaxQuantitySetting:
                            AddWarningSafe(Warnings,
                                ValidationIssue.RemovedDueToInvalidMaxQuantitySetting.ToString(),
                                $"The product {lineItemCode} does not have a valid max quantity setting and was removed from your cart.");
                            break;

                        case ValidationIssue.AdjustedQuantityByMinQuantity:
                            AddWarningSafe(Warnings,
                                ValidationIssue.AdjustedQuantityByMinQuantity.ToString(),
                                $"The quantity of product {lineItemCode} has changed by Min Quantity setting.");
                            break;

                        case ValidationIssue.AdjustedQuantityByMaxQuantity:
                            AddWarningSafe(Warnings,
                                ValidationIssue.AdjustedQuantityByMaxQuantity.ToString(),
                                $"The quantity of product {lineItemCode} has changed by Max Quantity setting.");
                            break;

                        case ValidationIssue.AdjustedQuantityByBackorderQuantity:
                            AddWarningSafe(Warnings,
                                ValidationIssue.AdjustedQuantityByBackorderQuantity.ToString(),
                                $"The quantity of product {lineItemCode} has changed by Backorder setting.");
                            break;

                        case ValidationIssue.AdjustedQuantityByPreorderQuantity:
                            AddWarningSafe(Warnings,
                                ValidationIssue.AdjustedQuantityByPreorderQuantity.ToString(),
                                $"The quantity of product {lineItemCode} has changed by Preorder setting.");
                            break;

                        case ValidationIssue.AdjustedQuantityByAvailableQuantity:
                            AddWarningSafe(Warnings,
                                ValidationIssue.AdjustedQuantityByAvailableQuantity.ToString(),
                                $"The quantity of product {lineItemCode} has changed by current available quantity.");
                            break;

                        case ValidationIssue.PlacedPricedChanged:
                            AddWarningSafe(Warnings,
                                ValidationIssue.PlacedPricedChanged.ToString(),
                                $"The price for product {lineItemCode} has changed since it was added to your cart.");
                            break;

                        default:
                            AddWarningSafe(Warnings,
                                lineItemCode,
                                $"There was some issue with product {lineItemCode} in your cart.");
                            break;
                    }
                }
            }
        }

        protected bool InventoryTrackingEnabled(ILineItem lineItem) => 
            lineItem.InventoryTrackingStatus == InventoryTrackingStatus.Enabled;

        protected InventoryTrackingStatus GetInventoryStatus(string entryCode, string warehouseCode)
        {
            var inventoryRecord = InventoryService.Get(entryCode, warehouseCode);
            if (inventoryRecord == null)
            {
                return InventoryTrackingStatus.Disabled;
            }

            return inventoryRecord.IsTracked ? InventoryTrackingStatus.Enabled : InventoryTrackingStatus.Disabled;
        }
    }
}