using EPiServer.Commerce.Order;
using Mediachase.Commerce.Customers;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.WorkflowCompatibility;
using System.Collections.Generic;
using System.Linq;

namespace Mediachase.Commerce.Workflow.Activities
{
    public class ValidateLineItemsActivity : OrderGroupActivityBase
    {
        /// <summary>
        /// Called by the workflow runtime to execute an activity.
        /// </summary>
        /// <param name="executionContext">The <see cref="T:Mediachase.Commerce.WorkflowCompatibility.ActivityExecutionContext"/> to associate with this <see cref="T:Mediachase.Commerce.WorkflowCompatibility.Activity"/> and execution.</param>
        /// <returns>
        /// The <see cref="T:Mediachase.Commerce.WorkflowCompatibility.ActivityExecutionStatus"/> of the run task, which determines whether the activity remains in the executing state, or transitions to the closed state.
        /// </returns>
        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            // Validate the properties at runtime
            ValidateRuntime();

            ValidateItems();

            // Return the closed status indicating that this activity is complete.
            return ActivityExecutionStatus.Closed;
        }

        private void ValidateItems()
        {
            var validationIssues = new Dictionary<ILineItem, IList<ValidationIssue>>();
            OrderGroup.ValidateOrRemoveLineItems((item, issue) => AddValidationIssues(validationIssues, item, issue),
                new[] { ValidationIssue.RemovedDueToCodeMissing });

            UpdateLineItemPrice();

            AddWarningMessages(validationIssues);
        }

        private void UpdateLineItemPrice()
        {
            var customerContact = CustomerContext.Current.GetContactById(OrderGroup.CustomerId);
            var billingCurrency = ((IOrderGroup)OrderGroup).Currency;

            foreach (var shipment in OrderGroup.OrderForms.SelectMany(form => form.Shipments))
            {
                var itemsToRemove = new List<ILineItem>();

                foreach (var lineItem in shipment.LineItems)
                {
                    var newListPrice = GetItemPrice(lineItem, customerContact, OrderGroup.BillingCurrency, OrderGroup.MarketId);

                    if (!newListPrice.HasValue)
                    {
                        AddWarningSafe(Warnings, "LineItemRemoved-" + lineItem.Code,
                            $"Item \"{lineItem.DisplayName}\" has been removed from the cart. The line item doesn't have price associated.");
                        itemsToRemove.Add(lineItem);
                    }

                    var lineItemInOrderForm = OrderGroup.OrderForms.SelectMany(f => f.LineItems).First(l => l.LineItemId == lineItem.LineItemId);
#pragma warning disable 618
                    var oldPrice = lineItemInOrderForm.ListPrice != 0 ? lineItemInOrderForm.ListPrice : lineItem.PlacedPrice;
#pragma warning restore 618

                    var oldListPrice = new Money(oldPrice, billingCurrency);

                    if (oldListPrice != newListPrice.Value)
                    {
                        if (oldListPrice > 0)
                        {
                            AddWarningSafe(Warnings,
                                "LineItemPriceChange-" + lineItem.Code,
                                $"Price for {lineItem.DisplayName} has been changed from {oldListPrice.ToString()} to {newListPrice.ToString()}.");
                        }

                        var newPrice = newListPrice.Value.Amount;
                        if (OrderGroup.ProviderId.ToLower().Equals("frontend") && OrderGroup is ICart)
                        {
                            lineItem.PlacedPrice = newPrice;
                        }

                        // Update list price for backward compatibility
#pragma warning disable 618
                        lineItemInOrderForm.ListPrice = newPrice;
                        if (lineItem is LineItem itemToUpdate)
                        {
                            itemToUpdate.ListPrice = newPrice;
                        }
#pragma warning restore 618
                    }
                }

                foreach (var lineItem in itemsToRemove)
                {
                    shipment.LineItems.Remove(lineItem);
                }
            }
        }
    }
}